// ignore_for_file: non_constant_identifier_names

import 'dart:convert';
import 'dart:math';
import 'dart:typed_data';

import 'package:encrypt/encrypt.dart';
import '../mcg_extension.dart';

class Crypt {
  Crypt._();

  /// [textKey] minimum 4 hane olmali
  static String? encryptMap(Map? data, String textKey) {
    if (data == null) return null;
    final encryptionPassword = _getModifiedPasswordFrom(textKey.safeSubString(0, 4)!);
    final iv = IV.fromUtf8(_getModifiedIvPasswordFrom(textKey));

    final encrypter = Encrypter(
      AES(Key.fromUtf8(encryptionPassword), mode: AESMode.cbc),
    );

    return 'enc001${(encrypter.encrypt(jsonEncode(data), iv: iv).base64)}';
  }

  static Map? decryptTextToMap(String? textData, String textKey) {
    if (textData == null) return null;
    if (!textData.startsWith('enc')) return null;

    final encryptionPassword = _getModifiedPasswordFrom(textKey.safeSubString(0, 4)!);

    final iv = IV.fromUtf8(_getModifiedIvPasswordFrom(textKey));
    final data = textData.substring(6);

    final encrypter = Encrypter(
      AES(
        Key.fromUtf8(encryptionPassword),
        mode: AESMode.cbc,
      ),
    );
    try {
      final decrptedData = encrypter.decrypt64(data, iv: iv);
      if (decrptedData is! String) return null;
      return json.decode(decrptedData) as Map?;
    } catch (err) {
      return null;
    }
  }

  static Uint8List encryptBytes(
    Uint8List byteData,
    String textKey,
  ) {
    final encryptionPassword = _getModifiedPasswordFrom(textKey.safeSubString(0, 4)!);
    final iv = IV.fromUtf8(_getModifiedIvPasswordFrom(textKey));
    final encrypter = Encrypter(AES(Key.fromUtf8(encryptionPassword), mode: AESMode.cbc));

    return (encrypter.encryptBytes(byteData, iv: iv).bytes);
  }

  static List<int> decryptBytes(
    Uint8List byteData,
    String textKey,
  ) {
    final encryptionPassword = _getModifiedPasswordFrom(textKey.safeSubString(0, 4)!);
    final iv = IV.fromUtf8(_getModifiedIvPasswordFrom(textKey));
    final encrypter = Encrypter(AES(Key.fromUtf8(encryptionPassword), mode: AESMode.cbc));

    return encrypter.decryptBytes(Encrypted(byteData), iv: iv);
  }

  static String _getModifiedPasswordFrom(String initialKey) {
    assert(initialKey.length <= 32);

    var paddingNeeded = 32 - initialKey.length;
    return paddingNeeded == 0 ? initialKey : initialKey + Get.appController.mapEncKey.substring(0, paddingNeeded);
  }

  static String _getModifiedIvPasswordFrom(String initialIV) {
    initialIV = initialIV.substring(0, min(10, initialIV.length));
    var paddingNeeded = 16 - initialIV.length;
    return initialIV + Get.appController.iVEncKey.substring(0, paddingNeeded);
  }
}

extension MapEncryptExtension on Map {
  String? encrypt(String IVKey) => Crypt.encryptMap(this, IVKey);
}

extension StringDecryptExtension on String {
  Map? decrypt(String IVKey) => Crypt.decryptTextToMap(this, IVKey);
}

extension UInt8ListEncryptExtension on Uint8List {
  Uint8List encrypt(String IVKey) => Crypt.encryptBytes(this, IVKey);
}

extension UInt8ListDecryptExtension on Uint8List {
  Uint8List decrypt(String IVKey) => Uint8List.fromList(Crypt.decryptBytes(this, IVKey));
}
