import 'package:flutter/material.dart';
import 'package:get/get.dart';

const _placeHolderColor = Colors.red;

class Design {
  final Color primary;
  final String? fontFamily;
  final Brightness brightness;
  final ScaffoldDesign scaffold;
  final SnackBarDesign snackBar;
  final ProgressIndicatorDesign progressIndicator;
  final DropdownDesign dropdown;
  final Color accent;
  final Color primaryText;
  final Color accentText;
  final Color disablePrimary;
  final BottomNavigationBarDesign bottomNavigationBar;
  final TextButtonDesign textButton;
  final ElevatedButtonDesign elevatedButton;
  final AppBarDesign appBar;
  final TextFieldDesign textField;
  final SheetDesign sheet;
  final ChipDesign chip;
  final SwitchDesign switchDesign;
  final CheckBoxDesign checkBox;
  final SlidingSegmentDesign slidingSegment;
  final EmptyStateDesign emptyState;
  final SelectableListItemDesign selectableListItem;
  final ListTileDesign listTile;
  final CardDesign card;

  final List<Color> customColors1;
  final List<Color> customColors2;
  final List<Color> customColors3;
  final List<Color> customColors4;
  final List<Color> customColors5;

  /////////////////

  final DesignSchema uploader;

  final DesignSchema customDesign1;
  final DesignSchema customDesign2;
  final DesignSchema customDesign3;
  final DesignSchema customDesign4;
  final DesignSchema customDesign5;

  final Map<String, Color> others;

  final int layoutStyle;
  final List<Gradient> gradientList;

  ColorScheme get colorScheme => Get.theme.colorScheme;

  const Design({
    required this.appBar,
    required this.accent,
    required this.accentText,
    required this.primary,
    required this.sheet,
    required this.primaryText,
    required this.brightness,
    required this.bottomNavigationBar,
    required this.progressIndicator,
    required this.scaffold,
    required this.snackBar,
    this.card = const CardDesign(),
    this.checkBox = const CheckBoxDesign(),
    this.chip = const ChipDesign(),
    this.customColors1 = const [_placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor],
    this.customColors2 = const [_placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor],
    this.customColors3 = const [_placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor],
    this.customColors4 = const [_placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor],
    this.customColors5 = const [_placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor, _placeHolderColor],
    this.customDesign1 = const DesignSchema(),
    this.customDesign2 = const DesignSchema(),
    this.customDesign3 = const DesignSchema(),
    this.customDesign4 = const DesignSchema(),
    this.customDesign5 = const DesignSchema(),
    this.disablePrimary = _placeHolderColor,
    this.dropdown = const DropdownDesign(),
    this.elevatedButton = const ElevatedButtonDesign(),
    this.emptyState = const EmptyStateDesign(),
    this.selectableListItem = const SelectableListItemDesign(),
    this.slidingSegment = const SlidingSegmentDesign(),
    this.switchDesign = const SwitchDesign(),
    this.textButton = const TextButtonDesign(),
    this.textField = const TextFieldDesign(),
    this.uploader = const DesignSchema(),
    this.others = const {},
    this.listTile = const ListTileDesign(),
    this.fontFamily,
    this.layoutStyle = 0,
    this.gradientList = const [
      LinearGradient(colors: [_placeHolderColor, _placeHolderColor]),
      LinearGradient(colors: [_placeHolderColor, _placeHolderColor]),
      LinearGradient(colors: [_placeHolderColor, _placeHolderColor])
    ],
  });
}

class DesignSchema {
  final Color background;
  final Color error;
  final Color primary;
  final Color onPrimary;
  final Color primaryVariant;
  final Color onPrimaryVariant;
  final Color accent;
  final Color onAccent;
  final Color secondaryVariant;
  final Color onSecondaryVariant;
  final Color disable;
  final Color onDisable;
  final Color canvas;
  final Color onCanvas;
  final String asset;
  final double assetOpacity;
  final Color border;
  final Color primaryText;
  final Color secondaryText;
  final Color shadow;

  const DesignSchema({
    this.asset = '',
    this.primary = _placeHolderColor,
    this.assetOpacity = 1.0,
    this.background = _placeHolderColor,
    this.canvas = _placeHolderColor,
    this.disable = _placeHolderColor,
    this.error = _placeHolderColor,
    this.onCanvas = _placeHolderColor,
    this.onDisable = _placeHolderColor,
    this.onPrimary = _placeHolderColor,
    this.onPrimaryVariant = _placeHolderColor,
    this.onAccent = _placeHolderColor,
    this.onSecondaryVariant = _placeHolderColor,
    this.primaryVariant = _placeHolderColor,
    this.accent = _placeHolderColor,
    this.secondaryVariant = _placeHolderColor,
    this.border = _placeHolderColor,
    this.primaryText = _placeHolderColor,
    this.secondaryText = _placeHolderColor,
    this.shadow = _placeHolderColor,
  });
}

class SnackBarDesign {
  final Color background;
  final Color dangerBackground;
  final Color text;

  const SnackBarDesign({this.background = Colors.black, this.dangerBackground = Colors.red, this.text = Colors.white});
}

class ScaffoldDesign {
  final Color background;
  final Color accentBackground;
  final String? backgroundAsset;
  final double backgroundAssetOpacity;

  const ScaffoldDesign({this.background = Colors.black, this.backgroundAsset, this.backgroundAssetOpacity = 0.0, this.accentBackground = _placeHolderColor});
}

class ProgressIndicatorDesign {
  final Color indicator;
  const ProgressIndicatorDesign({this.indicator = _placeHolderColor});
}

class AppBarDesign {
  final Color background;
  final Color accentBackground;
  final Color accentText;
  final Color text;
  const AppBarDesign({this.background = Colors.black, this.text = Colors.white, this.accentText = _placeHolderColor, this.accentBackground = _placeHolderColor});
}

class DropdownDesign {
  final Color border;
  final Color hint;
  final Color text;
  final Color canvas;
  final Color background;
  const DropdownDesign({this.border = _placeHolderColor, this.canvas = Colors.black, this.text = Colors.white, this.hint = _placeHolderColor, this.background = _placeHolderColor});
}

class BottomNavigationBarDesign {
  final Color background;
  final Color indicator;
  final Color selected;
  final Color unSelected;
  const BottomNavigationBarDesign({
    this.background = Colors.black,
    this.indicator = _placeHolderColor,
    this.selected = Colors.white,
    this.unSelected = _placeHolderColor,
  });
}

class TextButtonDesign {
  final Color text;

  const TextButtonDesign({
    this.text = _placeHolderColor,
  });
}

class ElevatedButtonDesign {
  final Color text;
  final Color background;
  final Color variantBackground;

  const ElevatedButtonDesign({
    this.text = Colors.white,
    this.background = _placeHolderColor,
    this.variantBackground = _placeHolderColor,
  });
}

class TextFieldDesign {
  final Color text;
  final Color border;
  final Color hint;
  final Color fill;

  const TextFieldDesign({
    this.text = Colors.white,
    this.border = _placeHolderColor,
    this.hint = _placeHolderColor,
    this.fill = Colors.black,
  });
}

class SheetDesign {
  final Color headerText;
  final Color contentText;
  final Color itemText;

  const SheetDesign({
    this.headerText = _placeHolderColor,
    this.contentText = _placeHolderColor,
    this.itemText = _placeHolderColor,
  });
}

class ChipDesign {
  final Color text;
  final Color background;

  const ChipDesign({
    this.text = Colors.white,
    this.background = _placeHolderColor,
  });
}

class SlidingSegmentDesign {
  final Color text;
  final Color background;

  const SlidingSegmentDesign({
    this.text = Colors.white,
    this.background = Colors.black,
  });
}

class SelectableListItemDesign {
  final Color unselected;
  final Color unselectedText;
  final Color selectedText;
  final Color selected;

  const SelectableListItemDesign({
    this.selected = _placeHolderColor,
    this.unselected = _placeHolderColor,
    this.selectedText = Colors.white,
    this.unselectedText = _placeHolderColor,
  });
}

class SwitchDesign {
  final Color color;
  final Color text;

  const SwitchDesign({
    this.color = Colors.black,
    this.text = Colors.white,
  });
}

class CheckBoxDesign {
  final Color checked;
  final Color unChecked;
  final Color text;

  const CheckBoxDesign({
    this.checked = _placeHolderColor,
    this.unChecked = _placeHolderColor,
    this.text = Colors.white,
  });
}

class EmptyStateDesign {
  final Color text;
  final String? noRecordsAsset;

  const EmptyStateDesign({
    this.text = Colors.white,
    this.noRecordsAsset,
  });
}

class CardDesign {
  final Color text;
  final Color variantText;
  final Color background;
  final Color button;
  final Color buttonText;

  const CardDesign({
    this.text = Colors.white,
    this.variantText = Colors.white,
    this.background = Colors.black,
    this.button = _placeHolderColor,
    this.buttonText = Colors.white,
  });
}

class ListTileDesign {
  final Color title;
  final Color subTitle;

  const ListTileDesign({
    this.title = _placeHolderColor,
    this.subTitle = _placeHolderColor,
  });
}
