import 'dart:io';

import 'package:device_info_plus/device_info_plus.dart';
import 'package:flutter/foundation.dart';

bool get isReleaseMode => kReleaseMode;
bool get isDebugMode => !kReleaseMode;

const isWeb = kIsWeb;
bool get isMobile => !kIsWeb;
bool get isAndroid => isMobile && Platform.isAndroid;
bool get isIOS => isMobile && Platform.isIOS;
String get platformName => isWeb
    ? 'Web'
    : isAndroid
        ? 'Android'
        : 'IOS';

class DeviceManager {
  DeviceManager._();

  static Future<String> iosSystemVersion() async {
    final _iosInfo = await DeviceInfoPlugin().iosInfo;
    return _iosInfo.systemVersion ?? '14.0.0';
  }

  static Future<bool> iosVersionGreater14() async {
    final _systemVersion = await iosSystemVersion();
    final _splittedVersion = _systemVersion.split('.');
    final int? _version1 = int.tryParse(_splittedVersion.first.trim());
    if (_version1 == null) return true;
    return _version1 >= 14;
  }

  static Future<bool> iosVersionGreater143() async {
    final _systemVersion = await iosSystemVersion();
    final _splittedVersion = _systemVersion.split('.');
    final int? _version1 = int.tryParse(_splittedVersion.first.trim());
    if (_version1 == null) return true;
    if (_version1 < 14) return false;
    if (_version1 > 14) return true;
    if (_splittedVersion.length < 2) return true;
    final int? _version2 = int.tryParse(_splittedVersion[1].trim());
    if (_version2 == null) return true;
    return _version2 > 2;
  }

  // ios => 13.5 icin 13 gibi
  static Future<int> iosMainSystemNo() async {
    final _systemVersion = await iosSystemVersion();
    final _splittedVersion = _systemVersion.split('.');
    return int.tryParse(_splittedVersion.first.trim()) ?? 14;
  }

  static Future<String> getDeviceModel() async {
    final _deviceInfoPlugin = DeviceInfoPlugin();
    late String _model;
    if (isWeb) {
      var _deviceInfo = await _deviceInfoPlugin.webBrowserInfo;
      _model = _deviceInfo.browserName.toString().split('.').last;
    } else if (Platform.isAndroid) {
      var _deviceInfo = await _deviceInfoPlugin.androidInfo;
      _model = _deviceInfo.model.toString() + _deviceInfo.device.toString();
    } else if (Platform.isIOS) {
      var _deviceInfo = await _deviceInfoPlugin.iosInfo;
      _model = _deviceInfo.utsname.machine ?? 'iosMachine';
    }
    return _model;
  }
}
