import 'package:avatar_glow/avatar_glow.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:hive/hive.dart';

import '../get/app_controller.dart';

extension MyGetExtension on GetInterface {
  Future<Box> openSafeBox(String boxKey) async {
    try {
      return await Hive.openBox(boxKey, encryptionCipher: HiveAesCipher(appController.secureKey));
    } catch (e) {
      await Hive.deleteBoxFromDisk(boxKey);
      return Hive.openBox(boxKey, encryptionCipher: HiveAesCipher(appController.secureKey));
    }
  }

  AppController get appController => Get.find<AppController>();
}

class MyFlareIndicator extends StatelessWidget {
  final String? text;
  final bool isCentered;
  final Color glowColor;
  final Widget? child;
  final bool isCircularProgressIndicatorActive;

  MyFlareIndicator({
    this.isCentered = true,
    this.text,
    this.child,
    this.glowColor = Colors.white,
    this.isCircularProgressIndicatorActive = true,
  });

  @override
  Widget build(BuildContext context) {
    var current = child ?? const SizedBox();

    if (isCircularProgressIndicatorActive) {
      current = Stack(
        alignment: Alignment.center,
        children: <Widget>[
          const SizedBox(
            width: 48,
            height: 48,
            child: CircularProgressIndicator(
              valueColor: AlwaysStoppedAnimation(Colors.white),
              strokeWidth: 2,
            ),
          ),
          current
        ],
      );
    }

    current = Container(
      width: 64,
      height: 64,
      decoration: BoxDecoration(
          boxShadow: [BoxShadow(color: const Color(0xff36D1DC).withAlpha(100))],
          shape: BoxShape.circle,
          gradient: LinearGradient(
            colors: [const Color(0xff36D1DC).withAlpha(150), const Color(0xff5B86E5).withAlpha(150)],
            begin: Alignment.topLeft,
            end: Alignment.bottomRight,
          )),
      child: current,
    );

    current = AvatarGlow(
      child: current,
      endRadius: 48,
      startDelay: 10.milliseconds,
      duration: 4.seconds,
      glowColor: glowColor,
    );

    if (text != null) {
      current = Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[current, const SizedBox(height: 8), Text(text!, textAlign: TextAlign.center, style: const TextStyle(color: Colors.white))],
      );
    }

    if (isCentered) current = Center(child: current);
    return current;
  }
}
