import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:mcg_extension/mcg_extension.dart';

class Sheet {
  Sheet._();
  static Future<bool> sure({String? message, String? title, String? cancelText}) async {
    var value = await make<bool>(
        title: title ?? 'sure'.translate,
        message: message,
        actions: [
          SheetAction(
            onPressed: () {
              Get.back(result: true);
            },
            title: 'yes'.translate,
            isDefaultAction: true,
          )
        ],
        cancelAction: SheetAction(
          onPressed: () {
            Get.back(result: false);
          },
          title: cancelText ?? 'back'.translate,
          isDestructiveAction: true,
        ));
    return value ?? false;
  }

  static Future<T?> make<T>({
    String? title,
    Widget? titleWidget,
    String? message,
    Widget? messageWidget,
    required List<SheetAction> actions,
    SheetAction? cancelAction,
  }) async {
    var result = await Get.bottomSheet(
      Theme(
        data: ThemeData(brightness: Brightness.dark),
        child: WillPopScope(
          onWillPop: () async {
            return false;
          },
          child: Container(
            padding: EdgeInsets.only(top: Get.context!.mediaQueryPadding.top),
            alignment: Alignment.bottomCenter,
            child: CupertinoActionSheet(
              title: titleWidget ?? title?.text.bold.make(),
              message: messageWidget ?? message?.text.make(),
              actions: actions
                  .map((e) => CupertinoActionSheetAction(
                        child: e.child ?? e.title.text.make(),
                        onPressed: e.onPressed,
                        isDestructiveAction: e.isDestructiveAction,
                        isDefaultAction: e.isDefaultAction,
                      ))
                  .toList(),
              cancelButton: cancelAction == null
                  ? null
                  : CupertinoActionSheetAction(
                      child: cancelAction.child ?? cancelAction.title.text.make(),
                      onPressed: cancelAction.onPressed,
                      isDestructiveAction: cancelAction.isDestructiveAction,
                      isDefaultAction: cancelAction.isDefaultAction,
                    ),
            ),
          ),
        ),
      ),
      isScrollControlled: true,
      isDismissible: false,
      enableDrag: false,
    );
    return result as T?;
  }
}

class SheetAction {
  final Function() onPressed;
  final bool isDefaultAction;
  final bool isDestructiveAction;
  final Widget? child;
  final String? title;

  SheetAction({required this.onPressed, this.isDefaultAction = false, this.isDestructiveAction = false, this.title, this.child});
  SheetAction.cancel()
      : onPressed = Get.back,
        isDefaultAction = false,
        isDestructiveAction = true,
        child = null,
        title = 'cancel'.translate;
}
