import '../extensions/stringextension.dart';

class J {
  J._();

  static int? jInt(dynamic data, [int? defaultValue]) {
    if (data == null) return defaultValue;
    if (data is int) return data;
    if (data is num) return data.toInt();
    if (data is String) return int.tryParse(data) ?? defaultValue;
    return defaultValue;
  }

  static double? jDouble(dynamic data, [double? defaultValue]) {
    if (data == null) return defaultValue;
    if (data is double) return data;
    if (data is num) return data.toDouble();
    if (data is String) return double.tryParse(data) ?? defaultValue;
    return defaultValue;
  }

  static String jString(dynamic data, [String? defaultValue]) {
    if (data == null) return defaultValue!;
    if (data is String) return data;
    return data.toString();
  }

  static T? jEnum<T>(dynamic data, List<T?> enumValues, [T? defaultValue]) {
    if (data == null) return defaultValue;
    if (data is! String) return defaultValue;
    return enumValues.firstWhere((element) => element.toString().afterDot == data,
        orElse: defaultValue == null
            ? null
            : () {
                return defaultValue;
              });
  }

  static List<T>? jClassList<T>(dynamic data, T Function(dynamic) parseFunction, [List<T>? defaultValue]) {
    if (data == null) return defaultValue;
    if (data is! List) return defaultValue;

    return data.map<T>(parseFunction).toList();
  }

  static List<T>? jEnumList<T>(dynamic data, List<T?> enumValues, [List<T>? defaultValue]) {
    if (data == null) return defaultValue;
    if (data is! List) return defaultValue;
    return data.fold<List<T>>(<T>[], (p, e) {
      final _enum = jEnum(e, enumValues);
      if (_enum == null) return p;
      return p..add(_enum);
    });
  }
}
//Haata veriyor
// extension JSsonMapExtension on dynamic {
//   int jInt([int defaultValue]) {
//     if (this == null) return defaultValue;
//     if (this is int) return this;
//     if (this is num) return (this as num).toInt();
//     if (this is String) return int.tryParse((this as String)) ?? defaultValue;
//     return defaultValue;
//   }

//   double jDouble([double defaultValue]) {
//     if (this == null) return defaultValue;
//     if (this is double) return this;
//     if (this is num) return (this as num).toDouble();
//     if (this is String) return double.tryParse((this as String)) ?? defaultValue;
//     return defaultValue;
//   }

//   String jString([String defaultValue]) {
//     if (this == null) return defaultValue;
//     if (this is String) return this;
//     return toString();
//   }

//   T jEnum<T>(List<T> enumValues, [T defaultValue]) {
//     if (this == null) return defaultValue;
//     if (this is! String) return defaultValue;
//     return enumValues.firstWhere((element) => element.toString().afterDot == this, orElse: () => defaultValue);
//   }
// }
