import 'package:flutter/material.dart';

extension MyIconWidget on IconData {
  IconBuilder get icon => IconBuilder(this);
}

@protected
class IconBuilder {
  IconBuilder(this._iconData);

  final IconData _iconData;
  Color? _iconColor;
  Color? _hoverColor;
  double? _iconSize;
  Function()? _onTap;
  double? _padding;

  IconBuilder size(double size) => this.._iconSize = size;
  IconBuilder color(Color color) => this.._iconColor = color;
  IconBuilder hoverColor(Color color) => this.._hoverColor = color;
  IconBuilder onPressed(VoidCallback onPressed) => this.._onTap = onPressed;
  IconBuilder padding(double size) => this.._padding = size;

  Widget make({Key? key}) {
    Widget _current = Icon(
      _iconData,
      color: _iconColor,
      size: _iconSize,
    );

    if (_onTap != null) {
      _current = IconButton(
        onPressed: _onTap,
        icon: _current,
        hoverColor: _hoverColor,
        visualDensity: VisualDensity.compact,
        padding: EdgeInsets.all(_padding ?? 8),
        constraints: BoxConstraints(maxWidth: (_padding ?? 8) * 2 + (_iconSize ?? 24), maxHeight: (_padding ?? 8) * 2 + (_iconSize ?? 24)),
      );
    } else {
      return Container(
        padding: EdgeInsets.all(_padding ?? 8),
        constraints: BoxConstraints(maxWidth: (_padding ?? 8) * 2 + (_iconSize ?? 24), maxHeight: (_padding ?? 8) * 2 + (_iconSize ?? 24)),
        alignment: Alignment.center,
        child: _current,
      );
    }

    return _current;
  }
}
