import 'package:auto_size_text/auto_size_text.dart';
import 'package:flutter/material.dart';
import '../get/translate.dart';

extension MyTextWidget on String? {
  TextBuilder get text => TextBuilder(this!);
}

@protected
class TextBuilder {
  TextBuilder(this._text);

  bool _autoSize = false;
  Color? _textColor;
  String _text;
  String? _fontFamily;

  double? _fontSize, _letterSpacing, _wordSpacing;
  int? _maxLines;
  FontWeight? _fontWeight;
  TextAlign? _textAlign;
  FontStyle? _fontStyle;
  TextDecoration? _decoration;

  StrutStyle? _strutStyle;
  TextOverflow? _overflow;
  TextBaseline? _textBaseline;

  bool? _softWrap;

  TextBuilder get tr => this.._text = _text.translate;

  TextBuilder fontSize(double size) => this.._fontSize = size;

  TextBuilder color(Color color) => this.._textColor = color;

  TextBuilder fontFamily(String family) => this.._fontFamily = family;

  TextBuilder maxLines(int lines) => this.._maxLines = lines;

  TextBuilder softWrap(bool softWrap) => this.._softWrap = softWrap;

  TextBuilder textBaseLine(TextBaseline baseline) => this.._textBaseline = baseline;

  TextBuilder wordSpacing(double spacing) => this.._wordSpacing = spacing;

  TextBuilder overflow(TextOverflow overflow) => this.._overflow = overflow;

  TextBuilder strutStyle(StrutStyle strutStyle) => this.._strutStyle = strutStyle;

  TextBuilder get center => this.._textAlign = TextAlign.center;
  TextBuilder get start => this.._textAlign = TextAlign.start;
  TextBuilder get end => this.._textAlign = TextAlign.end;
  TextBuilder get justify => this.._textAlign = TextAlign.justify;

  TextBuilder get fade => this.._overflow = TextOverflow.fade;
  TextBuilder get ellipsis => this.._overflow = TextOverflow.ellipsis;
  TextBuilder get visible => this.._overflow = TextOverflow.visible;

  TextBuilder get bold => this.._fontWeight = FontWeight.bold;
  TextBuilder setBold(bool value) => this.._fontWeight = value ? FontWeight.bold : FontWeight.normal;

  TextBuilder get italic => this.._fontStyle = FontStyle.italic;

  TextBuilder letterSpacing(double val) => this.._letterSpacing = val;

  TextBuilder get underline => this.._decoration = TextDecoration.underline;
  TextBuilder get lineThrough => this.._decoration = TextDecoration.lineThrough;
  TextBuilder get overline => this.._decoration = TextDecoration.overline;

  TextBuilder get autoSize => this.._autoSize = true;

  Widget make({Key? key}) {
    if (_autoSize) {
      return AutoSizeText(
        _text,
        key: key,
        textAlign: _textAlign,
        maxLines: _maxLines,
        softWrap: _softWrap ?? true,
        overflow: _overflow ?? TextOverflow.clip,
        strutStyle: _strutStyle,
        minFontSize: 8,
        style: TextStyle(
          color: _textColor,
          fontSize: _fontSize,
          fontStyle: _fontStyle,
          fontFamily: _fontFamily,
          fontWeight: _fontWeight,
          letterSpacing: _letterSpacing,
          decoration: _decoration,
          textBaseline: _textBaseline ?? TextBaseline.alphabetic,
          wordSpacing: _wordSpacing,
        ),
      );
    }
    return Text(
      _text,
      key: key,
      textAlign: _textAlign,
      maxLines: _maxLines,
      softWrap: _softWrap ?? true,
      overflow: _overflow ?? TextOverflow.clip,
      strutStyle: _strutStyle,
      style: TextStyle(
        color: _textColor,
        fontSize: _fontSize,
        fontStyle: _fontStyle,
        fontFamily: _fontFamily,
        fontWeight: _fontWeight,
        letterSpacing: _letterSpacing,
        decoration: _decoration,
        textBaseline: _textBaseline ?? TextBaseline.alphabetic,
        wordSpacing: _wordSpacing,
      ),
    );
  }
}
