import 'dart:convert';

import 'package:mcg_extension/preferences/i_preferences.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../mcg_extension.dart';

class MyPreferences {
  late SharedPreferences _preferences;

  MyPreferences();

  Future<void> init() async {
    _preferences = await SharedPreferences.getInstance();
  }

  Future<bool> remove(String key) {
    return _preferences.remove(key);
  }

  Future<bool> clear() {
    return _preferences.clear();
  }

  String? getString(String key, [String? defaultValue]) {
    return _preferences.getString(key) ?? defaultValue;
  }

  Future<bool> setString(String key, String value) {
    return _preferences.setString(key, value);
  }

  List<String>? getStringList(String key, [List<String>? defaultValue]) {
    return _preferences.getStringList(key) ?? defaultValue;
  }

  Future<bool> setStringList(String key, List<String> value) {
    return _preferences.setStringList(key, value);
  }

  Future<bool> removeThisStringInStringList(String key, String text) async {
    var _list = getLimitedStringList(key, [])!;

    if (_list.contains(text)) {
      _list.remove(text);
    }
    return setStringList('$limitListText$key', _list);
  }

  Future<bool> addLimitedStringList(String key, String text, {int maxNumber = 5}) async {
    var _list = getLimitedStringList(key, [])!;

    if (!_list.contains(text)) {
      _list.insert(0, text);
    }
    while (_list.length > maxNumber) {
      _list.removeLast();
    }
    return setStringList('$limitListText$key', _list);
  }

  List<String>? getLimitedStringList(String key, [List<String>? defaultValue]) {
    return _preferences.getStringList('$limitListText$key') ?? defaultValue;
  }

  bool? getBool(String key, [bool? defaultValue]) {
    return _preferences.getBool(key) ?? defaultValue;
  }

  Future<bool> setBool(String key, bool value) {
    return _preferences.setBool(key, value);
  }

  int? getInt(String key, [int? defaultValue]) {
    return _preferences.getInt(key) ?? defaultValue;
  }

  Future<bool> setInt(String key, int value) {
    return _preferences.setInt(key, value);
  }

  double? getDouble(String key, [double? defaultValue]) {
    return _preferences.getDouble(key) ?? defaultValue;
  }

  Future<bool> setDouble(String key, double value) {
    return _preferences.setDouble(key, value);
  }

  Future<Map> getHiveMap(boxName) async {
    var _box = await Get.openSafeBox(boxName);
    final Map _data = Map.from(_box.toMap());
    return _data;
  }

  Future setHiveMap(String boxName, Map data, {bool clearExistingData = false}) async {
    var _box = await Get.openSafeBox(boxName);
    if (clearExistingData) await _box.clear();
    await _box.putAll(data);
  }

  Map<String, dynamic> getMap(String key) {
    return Map<String, dynamic>.from(jsonDecode(getString(key).unMix ?? {}.toString()));
  }

  Future<void> setMap(String key, Map value) {
    return setString(key, json.encode(value).mix!);
  }

  Future<void> addItemToMap(String mapKey, String itemKey, dynamic value) async {
    final _existingValue = getMap(mapKey);
    _existingValue[itemKey] = value;
    await setString(mapKey, json.encode(_existingValue).mix!);
  }

  Future<void> deleteItemMap(String mapKey, String itemKey) async {
    final _existingValue = getMap(mapKey);
    _existingValue.remove(itemKey);
    await setString(mapKey, json.encode(_existingValue).mix!);
  }
}
