import 'package:hive_flutter/hive_flutter.dart';

import '../mcg_extension.dart';
import 'i_preferences.dart';

class SecurePreferences {
  late Box _secureBox;

  Future<void> init({int boxNo = 1}) async {
    _secureBox = await Get.openSafeBox('SecurePreferences$boxNo');
  }

  Future<void> remove(String key) {
    return _secureBox.delete(key);
  }

  Future<void> clear() {
    return _secureBox.clear();
  }

  String? getString(String key, [String? defaultValue]) {
    return _secureBox.get(key) ?? defaultValue;
  }

  Future<void> setString(String key, String value) {
    return _secureBox.put(key, value);
  }

  List<String>? getStringList(String key, [List<String>? defaultValue]) {
    return _secureBox.get(key) ?? defaultValue;
  }

  Future<void> setStringList(String key, List<String> value) {
    return _secureBox.put(key, value);
  }

  Future<void> removeThisStringInStringList(String key, String text) async {
    var _list = getStringList(key, [])!;

    if (_list.contains(text)) {
      _list.remove(text);
    }
    return setStringList(key, _list);
  }

  Future<void> addLimitedStringList(String key, String text, {int maxNumber = 5}) async {
    var _list = getLimitedStringList(key, [])!;

    if (!_list.contains(text)) {
      _list.insert(0, text);
    }
    while (_list.length > maxNumber) {
      _list.removeLast();
    }
    return setStringList('$limitListText$key', _list);
  }

  List<String>? getLimitedStringList(String key, [List<String>? defaultValue]) {
    return getStringList('$limitListText$key') ?? defaultValue;
  }

  bool? getBool(String key, [bool? defaultValue]) {
    return _secureBox.get(key) ?? defaultValue;
  }

  Future<void> setBool(String key, bool value) {
    return _secureBox.put(key, value);
  }

  int? getInt(String key, [int? defaultValue]) {
    return _secureBox.get(key) ?? defaultValue;
  }

  Future<void> setInt(String key, int value) {
    return _secureBox.put(key, value);
  }

  double? getDouble(String key, [double? defaultValue]) {
    return _secureBox.get(key) ?? defaultValue;
  }

  Future<void> setDouble(String key, double value) {
    return _secureBox.put(key, value);
  }

  Map<String, dynamic> getMap(String key, [Map<String, dynamic>? defaultValue]) {
    return Map<String, dynamic>.from(_secureBox.get(key) ?? defaultValue ?? {});
  }

  Future<void> setMap(String key, Map value) {
    return _secureBox.put(key, value);
  }

  Future<void> addItemToMap(String mapKey, String itemKey, dynamic value) async {
    final _existingValue = getMap(mapKey);
    _existingValue[itemKey] = value;
    await setMap(mapKey, _existingValue);
  }

  Future<void> deleteItemMap(String mapKey, String itemKey) async {
    final _existingValue = getMap(mapKey);
    _existingValue.remove(itemKey);
    await setMap(mapKey, _existingValue);
  }
}
